@@ -28,7 +28,7 @@ try:
 except ImportError:
     import importlib_metadata
 
-from pysrpm.convert import specifier_to_rpm_version, simplify_marker_to_rpm_condition
+from pysrpm.convert import specifier_to_rpm_version, simplify_marker_to_rpm_condition, python_version_to_rpm_version
 
 
 class RPMBuildError(Exception):
@@ -38,12 +38,13 @@ class RPMBuildError(Exception):
 
 class RPM:
     """ Given a python source distribution and a template config, build source, binary, or spec RPM files """
-    def __init__(self, source, config=None, **cli_args):
+    def __init__(self, source, config=None, templates={}, **options):
         """ Setup the various configurations and templates to start converting
 
         Args:
             source (`str` or path-like): Path to the source package to convert
             config (`str` or path-like): Path to a configuration file
+            templates (`dict`): CLI-specified template items
         """
         self.config = configparser.ConfigParser(interpolation=configparser.ExtendedInterpolation(),
                                                 dict_type=OrderedDict)
@@ -51,7 +52,8 @@ class RPM:
             self.config.read(config_lists)
 
         self.config_file = config
-        self.cli_args = cli_args
+        self.cli_options = options
+        self.cli_templates = templates
         source = pathlib.Path(source)
         if not source.exists():
             raise FileNotFoundError(str(source))
@@ -152,8 +154,8 @@ class RPM:
 
         # Load CLI last as it needs to override previous options.
         self.config.read_dict({
-            'pysrpm': {opt: arg for opt, arg in self.cli_args.items() if opt in options},
-            '__templates__': {opt: arg for opt, arg in self.cli_args.items() if opt not in options},
+            'pysrpm': {opt: arg for opt, arg in self.cli_options.items()},
+            '__templates__': {opt: arg for opt, arg in self.cli_templates.items()},
         }, source='CLI')
 
         # Set config params from loaded config
@@ -251,7 +253,7 @@ class RPM:
             except configparser.MissingSectionHeaderError:
                 pass
             entry_points = [importlib_metadata.EntryPoint(key, value, sec) for sec in ep_parser.sections()
-                            for key, value in parser.items(sec)]
+                            for key, value in ep_parser.items(sec)]
 
         pyproject = root / 'pyproject.toml'
         self.build_system = {
@@ -301,21 +303,24 @@ class RPM:
             else:
                 metadata[key] = f'{metadata[key]} {value}' if key in metadata else value
 
+        epoch, rpmversion = python_version_to_rpm_version(metadata['version']).rpartition(':')[::2]
         return {
             **metadata,
-            'rpmname': self.templates['package_prefix'] + re.sub('[._-]+', '-', metadata['name'].lower()),
-            'rpmversion': metadata['version'].replace('-', '_'),
+            'rpmname': self.templates['python_package'].format(name=re.sub('[._-]+', '-', metadata['name'].lower())),
+            'rpmversion': rpmversion,
             'release': self.config.get('pysrpm', 'release'),
             'arch': self.templates['arch'],
             'sourcefile': self.templates['source_name'].format(**metadata),
+            **({'epoch': epoch} if epoch != '' else {}),
         }
 
 
-    def convert_python_req(self, reqs, extras=[], python_package=None):
+    def convert_python_req(self, reqs, extras=[], package_template='python_dist'):
         """ Compute the version-specified dependency list for a package
 
         Args:
             reqs (`list` of `str`): The string representations of python dependencies
+            package_template (`str`): The name of the template to format package names
             extras (`list` of `str`): the list of extras to incldue in the requirement, to evaluate requirement markers
 
         Returns:
@@ -328,7 +333,7 @@ class RPM:
             if condition is False:
                 continue
 
-            package = (python_package or self.templates['python_package']).format(name=req.name)
+            package = self.templates[package_template].format(name=req.name)
             versioned_package = specifier_to_rpm_version(package, req.specifier)
 
             if condition is True:
@@ -372,9 +377,10 @@ class RPM:
             `str`: the contents of the spec file
         """
         spec = self._format_lines(self.templates['preamble'].lstrip('\n'), **pkg_info)
-        # In BuildRequires, python3 is not yet installed, ensure we do not need its version macro
+        # In BuildRequires, python3 is not yet installed, ensure we do not need its version macro by using a template
+        # that is not python-version dependent (as python_dist is). RPM should figure out what to do regardless.
         spec.append('BuildRequires: ' + ', '.join(self.convert_python_req(pkg_info['build-requires'],
-                                                                          python_package='python3-{name}')))
+                                                                          package_template='python_package')))
 
         # Handle automatically extracting dependencies
         extract_deps = self.config.getboolean('pysrpm', 'extract_dependencies')
